/*
	ef Toggle Lines/Curves v1.0
	evenflcw@yahoo.com

	"ef Toggle Lines/Curves" will convert selected (implicit or explicit)
	curves into 2-point polychains and vice-versa, similar to how
	native Toggle Faces/Subpatches toggles faces and subpatches (duh!).

	Log:
	2004XXXX v0.9	Initial build. Some minor flaws.
	20091223 v1.0   Someone actually needed it. Restructured and cleaned up.

	Possible improvements
		Multiple Layer support.
		Closed smooth curves from looping sets of polylines.
		Some clever way of handling crossing lines.
*/
@version 2.2
@warnings
@script modeler
@name "ef Toggle Lines/Curves"

@define STATE_INVALID_SELECTION		0
@define STATE_EXPLICIT_SELECTION	1
@define STATE_IMPLICIT_SELECTION	2

main
{
	state = GetState();
	if(state & (STATE_EXPLICIT_SELECTION+STATE_IMPLICIT_SELECTION)) {
		Process(state);
	}
	else
		error("No polygons or splines selected!");
}


GetState
{
	selmode(DIRECT);
	polyCount = polycount();
	if(polyCount[1]) {
		return STATE_EXPLICIT_SELECTION;
	}
	selmode(DIRECT);
	pointCount = pointcount();
	selmode(GLOBAL);
	polyCount = polycount();
	if(!pointCount && polyCount[1])
		return STATE_IMPLICIT_SELECTION;
	
	return STATE_INVALID_SELECTION;
}


Process: state
{
	selmode(USER);
	
	editbegin();
		foreach(poly, polygons)
		{
			if(poly.isCurve())
				curves += poly;
			else if(poly.pointCount == 2)
				lines += poly;
			 
		}
	editend();
	
	if(curves) {
		newLines = ConvertCurvesToLines(curves);
		if((state & STATE_EXPLICIT_SELECTION) && newLines.count())  selpolygon(SET, POLYID, newLines); //only reselects if an explicit selection was made
	}
	if(lines) {
		newCurves = ConvertLinesToCurves(lines);
		if((state & STATE_EXPLICIT_SELECTION) && newCurves.count()) selpolygon(SET, POLYID, newCurves); //only reselects if an explicit selection was made
	}
}



GetCurvesAndLines
{ //UNUSED, doesnt return arrays properly
	editend(); //if script crashed previously before ending meshedit, this closes the previous session
	editbegin();
		foreach(poly, polygons)
		{
			if(poly.isCurve())
				curves += poly;
			else if(poly.pointCount == 2)
				lines += poly;
			 
		}
	editend();
	
	return (curves, lines);
}



ConvertCurvesToLines: curves
{
    //converts curves to lines by creating polylines
    //from each pair of points in a curve.
	editbegin();
		foreach(curve, curves)
		{
			curvePoints = polyinfo(curve);
			pointCount = curve.pointCount;

			for(i=1; i<pointCount; i++)
			{
				pointPair[1] = curvePoints[i+1];
				pointPair[2] = curvePoints[i+2];
				lines += addpolygon(pointPair);
			}
			rempoly(curve);
		}
	editend();
	
    return lines;
}

ConvertLinesToCurves: Lines
{
    //convertes lines to curves by creating curves
	//from each pair of points specified by the 
	//polylines, then merging those curves.
	
	//no good way to handle crossing lines
	//user must make explicit poly selection
	
	editbegin();
		foreach(line, Lines)
		{
			LinePnts = polyinfo(line);

			// if(LinePnts[2].polygon > 2 || LinePnts[3].polygon > 2)
				// error("Bad geometry!");

			PntPair[1] = LinePnts[2];
			PntPair[2] = LinePnts[3];

			curves += addcurve(PntPair);

			rempoly(line);
		}
    editend();

    selpolygon(CLEAR, NVGT, 2);
    selpolygon(SET, POLYID, curves);
    mergepols();
    editbegin();
	curves = polygons; //grab merged curves instead
	editend();
	selpolygon(CLEAR);
	
	return curves;
}





